import org.hibernate.Session;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

import org.mikgul.Author;
import org.mikgul.AuthorBook;
import org.mikgul.Book;
import org.mikgul.Genre;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

public class TestAuthor {
    private Session session;

    @Before
    public void setUp() {
        try {
            EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("hibernate-dynamic");

            EntityManager entityManager = entityManagerFactory.createEntityManager();
            this.session = entityManager.unwrap(Session.class);
        } catch (Throwable ex) {
            System.err.println("Creating session failed." + ex);
        }
    }

    @After
    public void tearDown() {
        if (this.session != null) {
            this.session.close();
        }
    }

    @Test
    public void testCRUD() {
        Genre genre = new Genre("Thriller");
        Book book = new Book("Test title", false, genre);
        Author author = new Author("John", "Doe", LocalDate.of(1990, 1, 1), true);
        AuthorBook authorBook = new AuthorBook(author, book);

        // CREATE
        author.save(session);
        assertNotNull(author.getId());

        // READ
        Author retrievedAuthor = Author.getById(session, author.getId());
        assertNotNull(retrievedAuthor);
        assertEquals(author.getFirstName(), retrievedAuthor.getFirstName());
        assertEquals(author.getLastName(), retrievedAuthor.getLastName());
        assertEquals(author.getDateOfBirth(), retrievedAuthor.getDateOfBirth());
        assertEquals(author.isAlive(), retrievedAuthor.isAlive());
        assertTrue(retrievedAuthor.getAuthorBooks().contains(authorBook));

        // UPDATE
        retrievedAuthor.setFirstName("Jacob");
        retrievedAuthor.setLastName("Smith");
        retrievedAuthor.setAlive(false);
        retrievedAuthor.update(session);
        Author updatedAuthor = Author.getById(session, retrievedAuthor.getId());
        assertNotNull(updatedAuthor);
        assertEquals("Jacob", updatedAuthor.getFirstName());
        assertEquals("Smith", updatedAuthor.getLastName());
        assertFalse(updatedAuthor.isAlive());

        // DELETE
        author.delete(session);
        Author deletedAuthor = Author.getById(session, author.getId());
        assertNull(deletedAuthor);
    }

    @Test
    public void testGetAuthorsByLastName() {
        Author author1 = new Author("John", "Doe", LocalDate.of(1990, 1, 1), true);
        author1.save(session);
        Author author2 = new Author("Jacob", "Doe", LocalDate.of(1954, 12, 10), false);
        author2.save(session);
        Author author3 = new Author("Mary", "Sue", LocalDate.of(1823, 5, 13), false);
        author3.save(session);

        List<Author> authors = new ArrayList<>();
        authors.add(author1);
        authors.add(author2);
        authors.add(author3);

        List<Author> retrievedAuthors = Author.getAuthorsByLastName(session, "Doe");
        assertNotNull(retrievedAuthors);
        assertFalse(retrievedAuthors.isEmpty());
        for (Author ra : retrievedAuthors) {
            assertTrue(authors.contains(ra));
        }

        author3.setLastName("Doe");
        author3.update(session);

        retrievedAuthors = Author.getAuthorsByLastName(session, "Doe");
        assertNotNull(retrievedAuthors);
        assertEquals(retrievedAuthors, authors);

        author1.delete(session);
        author2.delete(session);
        author3.delete(session);

        retrievedAuthors = Author.getAuthorsByLastName(session, "Doe");
        assertNotNull(retrievedAuthors);
        assertTrue(retrievedAuthors.isEmpty());
    }

    @Test
    public void testGetAuthorsByLastNameEmpty() {
        Author author1 = new Author("John", "Doe", LocalDate.of(1990, 1, 1), true);
        author1.save(session);

        List<Author> retrievedAuthors = Author.getAuthorsByLastName(session, "Invalid");
        assertNotNull(retrievedAuthors);
        assertTrue(retrievedAuthors.isEmpty());

        author1.delete(session);
    }
}
