import org.hibernate.Session;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

import org.mikgul.*;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;

public class TestBestseller {
    private Session session;

    @Before
    public void setUp() {
        try {
            EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("hibernate-dynamic");

            EntityManager entityManager = entityManagerFactory.createEntityManager();
            this.session = entityManager.unwrap(Session.class);
        } catch (Throwable ex) {
            System.err.println("Creating session failed." + ex);
        }
    }

    @After
    public void tearDown() {
        if (this.session != null) {
            this.session.close();
        }
    }

    @Test
    public void testCRUD() {
        Genre genre = new Genre("Romance");
        genre.save(session);
        Book book = new Book("Test title", true, genre);
        book.save(session);
        Bestseller bestseller = new Bestseller(book,
                105134113L,
                (short) 97,
                true,
                ZonedDateTime.of(1990, 4, 5, 9, 0, 0, 0, ZoneId.of("+02:00")),
                null);

        // CREATE
        bestseller.save(session);
        assertNotNull(bestseller.getId());

        // READ
        Bestseller retrievedBestseller = Bestseller.getById(session, bestseller.getId());
        assertNotNull(retrievedBestseller);
        assertEquals(book, retrievedBestseller.getBook());
        assertEquals(bestseller.getCopiesSold(), retrievedBestseller.getCopiesSold());
        assertEquals(bestseller.getRating(), retrievedBestseller.getRating());
        assertEquals(bestseller.getAwards(), retrievedBestseller.getAwards());

        // UPDATE
        retrievedBestseller.setCopiesSold(611295875L);
        retrievedBestseller.setRating((short) 31);
        retrievedBestseller.setAwards(2);
        retrievedBestseller.update(session);
        Bestseller updatedBestseller = Bestseller.getById(session, retrievedBestseller.getId());
        assertNotNull(updatedBestseller);
        assertEquals(611295875L, updatedBestseller.getCopiesSold());
        assertEquals((short) 31, updatedBestseller.getRating());
        assertEquals((Integer) 2, updatedBestseller.getAwards());

        // DELETE
        bestseller.delete(session);
        Bestseller deletedBestseller = Bestseller.getById(session, bestseller.getId());
        assertNull(deletedBestseller);

        book.delete(session);
        genre.delete(session);
    }

    @Test
    public void testGetBestsellersWithHighRatings() {
        Genre genre = new Genre("Romance");
        genre.save(session);
        Book book1 = new Book("Test title 1", true, genre);
        book1.save(session);
        Book book2 = new Book("Test title 2", true, genre);
        book2.save(session);
        Book book3 = new Book("Test title 3", true, genre);
        book3.save(session);
        Bestseller bestseller1 = new Bestseller(book1, 105134113L, (short) 97);
        bestseller1.save(session);
        Bestseller bestseller2 = new Bestseller(book2, 412459582L, (short) 75);
        bestseller2.save(session);
        Bestseller bestseller3 = new Bestseller(book3, 9203112L, (short) 82);
        bestseller3.save(session);

        List<Bestseller> bestsellers = new ArrayList<>();
        bestsellers.add(bestseller1);
        bestsellers.add(bestseller2);
        bestsellers.add(bestseller3);

        List<Bestseller> retrievedBestsellers = Bestseller.getBestsellersWithHighRatings(session, (short) 80);
        assertNotNull(retrievedBestsellers);
        assertFalse(retrievedBestsellers.isEmpty());
        for (Bestseller rb : retrievedBestsellers) {
            assertTrue(bestsellers.contains(rb));
        }

        retrievedBestsellers = Bestseller.getBestsellersWithHighRatings(session, (short) 0);
        assertNotNull(retrievedBestsellers);
        assertEquals(retrievedBestsellers, bestsellers);

        bestseller1.delete(session);
        bestseller2.delete(session);
        bestseller3.delete(session);
        book1.delete(session);
        book2.delete(session);
        book3.delete(session);
        genre.delete(session);

        retrievedBestsellers = Bestseller.getBestsellersWithHighRatings(session, (short) 0);
        assertNotNull(retrievedBestsellers);
        assertTrue(retrievedBestsellers.isEmpty());
    }

    @Test
    public void testGetBestsellersWithHighRatingsError() {
        assertNull(Bestseller.getBestsellersWithHighRatings(session, (short) -1));
        assertNull(Bestseller.getBestsellersWithHighRatings(session, (short) 101));
    }
}