import org.hibernate.Session;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

import org.mikgul.Author;
import org.mikgul.AuthorBook;
import org.mikgul.Book;
import org.mikgul.Genre;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

public class TestBook {
    private Session session;

    @Before
    public void setUp() {
        try {
            EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("hibernate-dynamic");

            EntityManager entityManager = entityManagerFactory.createEntityManager();
            this.session = entityManager.unwrap(Session.class);
        } catch (Throwable ex) {
            System.err.println("Creating session failed." + ex);
        }
    }

    @After
    public void tearDown() {
        if (this.session != null) {
            this.session.close();
        }
    }

    @Test
    public void testCRUD() {
        Genre genre = new Genre("Thriller");
        genre.save(session);
        Book book = new Book("Test title", false, genre);
        Author author = new Author("John", "Doe", LocalDate.of(1900, 1, 1), false);
        AuthorBook authorBook = new AuthorBook(author, book);

        // CREATE
        book.save(session);
        assertNotNull(book.getId());

        // READ
        Book retrievedBook = Book.getById(session, book.getId());
        assertNotNull(retrievedBook);
        assertEquals(book.getId(), retrievedBook.getId());
        assertEquals(book.getTitle(), retrievedBook.getTitle());
        assertEquals(book.isBestseller(), retrievedBook.isBestseller());
        assertEquals(genre, retrievedBook.getGenre());
        assertTrue(retrievedBook.getAuthorBooks().contains(authorBook));

        // UPDATE
        retrievedBook.setTitle("Updated title");
        retrievedBook.setBestseller(true);
        retrievedBook.update(session);
        Book updatedBook = Book.getById(session, retrievedBook.getId());
        assertNotNull(updatedBook);
        assertEquals("Updated title", updatedBook.getTitle());
        assertTrue(updatedBook.isBestseller());

        // DELETE
        book.delete(session);
        Book deletedBook = Book.getById(session, book.getId());
        assertNull(deletedBook);

        genre.delete(session);
    }

    @Test
    public void testGetBooksPaginated() {
        Genre genre = new Genre("Thriller");
        genre.save(session);

        Book book1 = new Book("Test title 1", false, genre);
        book1.save(session);
        Book book2 = new Book("Test title 2", true, genre);
        book2.save(session);
        Book book3 = new Book("Test title 3", false, genre);
        book3.save(session);


        List<Book> books = new ArrayList<>();
        books.add(book1);
        books.add(book2);
        books.add(book3);

        // pierwsze dwa rekordy
        List<Book> retrievedBooks = Book.getBooksPaginated(session, 1, 2);
        assertNotNull(retrievedBooks);
        assertFalse(retrievedBooks.isEmpty());
        for (Book rb : retrievedBooks) {
            assertTrue(books.contains(rb));
        }

        // wszystkie rekordy
        retrievedBooks = Book.getBooksPaginated(session, 1, 3);
        assertNotNull(retrievedBooks);
        assertEquals(retrievedBooks, books);

        book1.delete(session);
        book2.delete(session);
        book3.delete(session);
        genre.delete(session);

        retrievedBooks = Book.getBooksPaginated(session, 1, 3);
        assertNotNull(retrievedBooks);
        assertTrue(retrievedBooks.isEmpty());
    }

    @Test
    public void testGetBooksPaginatedError() {
        assertNull(Book.getBooksPaginated(session, -2, 0));
        assertNull(Book.getBooksPaginated(session, 1, -5));
    }
}
